"""
Stub file.

You should never be importing ``nitrate`` directly, because it isn't
really a single package -- it's more a collection of different packages
distributed as one library.

You should always import individual files, e.g.

    from nitrate.json import DatetimeDecoder

This file only exists to make Python's package management tools happy,
and to hold the version number.
"""

__version__ = "1.2.0"
